const ElasticSearch = require("../ElasticSearch");
const ElasticClient = require("../ElasticClient");

class User extends ElasticSearch {
    constructor() {
        super();
    }

    /**
     * type
     * @return {string}
     */
    static get type() {
        return "user";
    }

    /**
     * index
     * @return {string}
     */
    static get index() {
        if(ElasticClient.prefix !== undefined && ElasticClient.prefix !== null && ElasticClient.prefix.length > 0) {
            return ElasticClient.prefix + "-share";
        } else {
            return "share";
        }
    }

    /**
     * Mapping information of the document share-user
     */
    static get mapping()
    {
        let properties = {
            id : {
                type : "integer",
            },
            tenant_id : {
                type : "integer",
            },
            firstname : {
                type : 'completion',
                analyzer : 'english'
            },
            lastname : {
                type : "text",
            },
            midname : {
                type : "text",
            },
            email : {
                type : "text",
            },
            status : {
                type : "integer",
            },
            advertiser_id : {
                type : "integer",
            },
            agency_id : {
                type : "integer",
            },
            avatar_id : {
                type : "text",
            },
            title : {
                type : "text",
            },
            last_login : {
                type : "long",
            },
            default : {
                type : "integer",
            },
            createdAt : {
                type : "long",
            },
            updatedAt : {
                type : "long",
            },
            token : {
                type : "text",
            },
            token_generated_time : {
                type : "long",
            },
            username : {
                type : "text",
                analyzer : 'english',
                search_analyzer : 'english',
                fields : {
                    raw : {
                        type : "keyword"
                    },
                    suggest : {
                        type : "completion",
                        analyzer : "simple",
                        search_analyzer: "simple"
                    }
                }
            }
        };
        let ret = {};
        ret[this.type] = { properties : properties};
        return ret;
    }

    /**
     * returns fields of primary keys
     * @returns {string[]}
     */
    static get primaryKeys() {
        return ["id"];
    }

    /**
     * returns elastic client
     * @return {es.Client|Client}
     */
    static get client() {
        return ElasticClient.client;
    }

    /**
     * Save user into elasticsearch
     * @static
     * @param data json data
     * @param refresh whether the change should be visible to search
     * @return {Promise<any>}
     * @example
     * let tmp = {
     *   id: 1,
     *   tenant_id : 1,
     *   firstname: 'first',
     *   lastname: 'last',
     *   email: "support@owcmvc.com",
     *   status : 1,
     *   default : 1,
     *   username : "owc"
     * }
     * User.saveUser(tmp, true);
     */
    static saveUser(data, refresh = false) {
        let obj = this._setAttributes(data);
        return obj.save(refresh);
    }



}

module.exports = User;