const test = require('unit.js');
const ElasticClient = require("../ElasticClient");
const User = require("./User");
let config = {
    hosts: ["127.0.0.1:9200"],
    apiVersion: "5.6",
    requestTimeout: 30000
};

ElasticClient.init(config);

it("test index exists expect false", function(done) {
    User.indexExists().then(data => {
        test.value(data).is(false);
        done(null);
    }).catch(err => {
        done(err);
    });
});

it("test create index", function(done) {
    User.createIndex().then(data => {
        test.value(data.acknowledged).is(true);
        done(null);
    }).catch(err => {
        done(err);
    });
});

it("test index exists expect true", function(done) {
    User.indexExists().then(data => {
        test.value(data).is(true);
        done(null);
    }).catch(err => {
        done(err);
    });
});

it("test save user", function(done) {
    let tmp = {
        id: 1,
        tenant_id : 1,
        firstname: 'first',
        lastname: 'last',
        email: "support@owcmvc.com",
        status : 1,
        default : 1,
        username : "owc"
    }
    User.saveUser(tmp, true).then(data => {
        console.log(data);
        test.value(data.created).is(true);
        done(null);
    }).catch(err => {
        done(err);
    });
});

it("test save user", function(done) {
    let tmp = {
        id: 2,
        tenant_id : 2,
        firstname: ['second', 'abcd', 'name'],
        lastname: 'second user last name',
        email: "support2@owcmvc.com",
        status : 2,
        default : 2,
        username : "test search function"
    }
    User.saveUser(tmp, true).then(data => {
        console.log(data);
        test.value(data.created).is(true);
        done(null);
    }).catch(err => {
        done(err);
    });
});

it("test update user", function(done) {
    let tmp = {
        id: 2,
        tenant_id : 3,
        firstname: ['second', 'abcd', 'name'],
        lastname: 'second user last name',
        email: "support2@owcmvc.com",
        status : 2,
        default : 2,
        username : "test search function"
    }
    User.saveUser(tmp, true).then(data => {
        console.log(data);
        test.value(data.result).is("updated");
        done(null);
    }).catch(err => {
        done(err);
    });
});

it("test user exists expect true", function(done) {
    User.exists(1).then(data => {
        console.log(data);
        test.value(data).is(true);
        done(null);
    }).catch(err => {
        done(err);
    });
});

it("get user", function(done) {
    User.find().filter({id:1}).one().then(data => {
        console.log(data);
        test.value(data.id).is(1);
        done(null);
    }).catch(err => {
        done(err);
    });
});

it("get all users", function(done) {
    User.find().all().then(data => {
        console.log(data);
        test.value(data.total).is(2);
        done(null);
    }).catch(err => {
        done(err);
    });
});

it("get all users that have status between 0 and 1", function(done) {
    User.find().filter({status : [0,1]}).all().then(data => {
        console.log(data);
        test.value(data.total).is(1);
        done(null);
    }).catch(err => {
        done(err);
    });
});

it("get all users that have status between 0 and 1 and default equals to 0", function(done) {
    User.find().filter({status : [0,1], default : 0}).all().then(data => {
        console.log(data);
        test.value(data.total).is(0);
        done(null);
    }).catch(err => {
        done(err);
    });
});

it("get all users that have status between 0 and 1 or default equals to 0", function(done) {
    User.find().filter({default: 2}).orFilter({status: [0,1]}).all().then(data => {
        console.log(data);
        test.value(data.total).is(2);
        done(null);
    }).catch(err => {
        done(err);
    });
});

it("get all users that have status between 0 and 1 or default equals to 0 and username has keyword 'test'", function(done) {
    User.find().filter({default: 2}).orFilter({status: [0,1]}).query({match : {"username" : 'test'}}).all().then(data => {
        console.log(data);
        test.value(data.total).is(1);
        done(null);
    }).catch(err => {
        done(err);
    });
});

it("get all users that have status between 0 and 1 and default equals to 2 and username has keyword 'test'", function(done) {
    User.find().filter({default: 2}).andFilter({status: [0,1]}).query({match : {"username" : 'test'}}).all().then(data => {
        console.log(data);
        test.value(data.total).is(0);
        done(null);
    }).catch(err => {
        done(err);
    });
});

it("get all users that have status in range 0 to 1 and default equals to 2 and username has keyword 'test'", function(done) {
    User.find().filter({default: 2}).range("status", {gte:0, lte : 1}).andQuery({match : {"username" : 'test'}}).all().then(data => {
        console.log(data);
        test.value(data.total).is(0);
        done(null);
    }).catch(err => {
        done(err);
    });
});

it("get all users whose username has keyword 'test' or 'owc'", function(done) {
    User.find().query([{match : {"username" : 'test'}} ]).orQuery([{match : {"username" : 'owc'}}]).all().then(data => {
        console.log(data);
        test.value(data.total).is(2);
        done(null);
    }).catch(err => {
        done(err);
    });
});

it("get all users whose username has keyword 'test' or 'owc'", function(done) {
    User.find().query([{match : {"username" : 'test'}}, {match : {"username" : 'owc'}}], "should").all().then(data => {
        console.log(data);
        test.value(data.total).is(2);
        done(null);
    }).catch(err => {
        done(err);
    });
});

it("get all users whose username has keyword 'test' and 'search'", function(done) {
    User.find().query([{match : {"username" : 'test'}}, {match : {"username" : 'search'}} ]).all().then(data => {
        console.log(data);
        test.value(data.data[0]['id']).is(2);
        done(null);
    }).catch(err => {
        done(err);
    });
});

it("get all users whose username contains 'sear'", function(done) {
    User.suggest(".*sear.*", "username.suggest", true).then(data => {
        console.log(JSON.stringify(data));
        test.value(data.length).is(1);
        done(null);
    }).catch(err => {
        done(err);
    });
});

it("get all users whose username contains 'tast' with fuzziness 2", function(done) {
    let options = {
        "fuzzy" : {
            "fuzziness" : 2
        }
    }
    User.suggest("tast", "username.suggest", false, ["id", "username"], 5, options).then(data => {
        console.log(JSON.stringify(data));
        test.value(data[0]._source.id).is(2);
        done(null);
    }).catch(err => {
        done(err);
    });
});

it("get all users whose username has keyword 'test' or 'owc' and doesn't have keyword 'search'", function(done) {
    User.find().query([{match : {"username" : 'test'}} ]).orQuery([{match : {"username" : 'owc'}}]).mustNotQuery([{match : {"username" : 'search'}}]).all().then(data => {
        console.log(data);
        test.value(data.total).is(1);
        done(null);
    }).catch(err => {
        done(err);
    });
});

it("get all users whose username has keyword 'test' or 'owc', lastname has keyword 'second user' and username doesn't have keyword 'search'", function(done) {
    User.find().query([{match : {"username" : 'test'}} ]).orQuery([{match : {"username" : 'owc'}}]).andQuery({match_phrase: {"lastname": "second user"}}).mustNotQuery([{match : {"username" : 'search'}}]).all().then(data => {
        console.log(data);
        test.value(data.total).is(0);
        done(null);
    }).catch(err => {
        done(err);
    });
});

it("get all users whose username has keyword 'second user'", function(done) {
    User.find().andQuery({match_phrase: {"lastname": "second user"}}).all().then(data => {
        console.log(data);
        test.value(data.data[0].id).is(2);
        done(null);
    }).catch(err => {
        done(err);
    });
});

it("get all users whose username has keyword 'test' or 'owc', lastname has keyword 'second' and username doesn't have keyword 'search'", function(done) {
    User.find().query([{match : {"username" : 'test'}} ]).orQuery([{match : {"username" : 'owc'}}]).andQuery({match: {"lastname": "second"}}).mustNotQuery([{match : {"username" : 'search'}}]).all().then(data => {
        console.log(data);
        test.value(data.total).is(0);
        done(null);
    }).catch(err => {
        done(err);
    });
});

it("Find users whose username like 'search fu'", function(done) {
    User.find().query([{match_phrase_prefix : {"username" : 'search fu'}}]).all().then(data => {
        console.log(data);
        test.value(data.data[0].id).is(2);
        done(null);
    }).catch(err => {
        done(err);
    });
});

it("Find users whose status between 0 and 2 and default equals 2", function(done) {
    User.find().range("status", {gte : 0, lte : 2}).andFilter({default : 2}).all().then(data => {
        console.log(data);
        test.value(data.data[0].id).is(2);
        done(null);
    }).catch(err => {
        done(err);
    });
});


it("delete user whose id is 1", function(done) {
    User.delete(1).then(data => {
        console.log(data);
        test.value(data.found).is(true);
        done(null);
    }).catch(err => {
        done(err);
    });
});

it("delete user whose username has keyword 'search'", function(done) {
    User.find().query({match:{username: 'search'}}).deleteAll().then(data => {
        console.log(data);
        test.value(data.deleted).is(1);
        done(null);
    }).catch(err => {
        done(err);
    });
});



it("delete index expect true", function(done) {
    User.deleteIndex().then(data => {
        console.log(JSON.stringify(data));
        test.value(data.acknowledged).is(true);
        done(null);
    }).catch(err => {
        done(err);
    });
});

